@@ -147,7 +147,7 @@ abstract class Stream<T> {
         if (--count == 0) controller._closeUnchecked();
       }
     };
-    var onError = (error, stack) {
+    var onError = (error, StackTrace stack) {
       if (!controller.isClosed) {
         controller._addError(error, stack);
         if (--count == 0) controller._closeUnchecked();
@@ -426,15 +426,14 @@ abstract class Stream<T> {
    * The returned stream is a broadcast stream if this stream is.
    */
   Stream<E> asyncMap<E>(FutureOr<E> convert(T event)) {
-    StreamController<E> controller;
+    _StreamControllerBase<E> controller;
     StreamSubscription<T> subscription;
 
     void onListen() {
       final add = controller.add;
-      assert(controller is _StreamController ||
+      assert(controller is _StreamController<E> ||
           controller is _BroadcastStreamController);
-      final _EventSink<E> eventSink = controller as Object;
-      final addError = eventSink._addError;
+      final addError = controller._addError;
       subscription = this.listen((T event) {
         FutureOr<E> newValue;
         try {
@@ -495,12 +494,11 @@ abstract class Stream<T> {
    * The returned stream is a broadcast stream if this stream is.
    */
   Stream<E> asyncExpand<E>(Stream<E> convert(T event)) {
-    StreamController<E> controller;
+    _StreamControllerBase<E> controller;
     StreamSubscription<T> subscription;
     void onListen() {
       assert(controller is _StreamController ||
           controller is _BroadcastStreamController);
-      final _EventSink<E> eventSink = controller as Object;
       subscription = this.listen((T event) {
         Stream<E> newStream;
         try {
@@ -514,7 +512,7 @@ abstract class Stream<T> {
           controller.addStream(newStream).whenComplete(subscription.resume);
         }
       },
-          onError: eventSink._addError, // Avoid Zone error replacement.
+          onError: controller._addError, // Avoid Zone error replacement.
           onDone: controller.close);
     }
 
@@ -1195,7 +1193,7 @@ abstract class Stream<T> {
    */
   Future<T> get last {
     _Future<T> future = new _Future<T>();
-    T result = null;
+    T result;
     bool foundResult = false;
     listen(
         (T value) {
@@ -1230,7 +1228,7 @@ abstract class Stream<T> {
    */
   Future<T> get single {
     _Future<T> future = new _Future<T>();
-    T result = null;
+    T result;
     bool foundResult = false;
     StreamSubscription subscription;
     subscription = this.listen(
@@ -1328,7 +1326,7 @@ abstract class Stream<T> {
    */
   Future<T> lastWhere(bool test(T element), {T orElse()}) {
     _Future<T> future = new _Future();
-    T result = null;
+    T result;
     bool foundResult = false;
     StreamSubscription subscription;
     subscription = this.listen(
@@ -1368,7 +1366,7 @@ abstract class Stream<T> {
    */
   Future<T> singleWhere(bool test(T element), {T orElse()}) {
     _Future<T> future = new _Future<T>();
-    T result = null;
+    T result;
     bool foundResult = false;
     StreamSubscription subscription;
     subscription = this.listen(
@@ -1425,7 +1423,8 @@ abstract class Stream<T> {
    * with a [RangeError].
    */
   Future<T> elementAt(int index) {
-    if (index is! int || index < 0) throw new ArgumentError(index);
+    ArgumentError.checkNotNull(index, "index");
+    RangeError.checkNotNegative(index, "index");
     _Future<T> future = new _Future<T>();
     StreamSubscription subscription;
     int elementIndex = 0;
@@ -1474,7 +1473,7 @@ abstract class Stream<T> {
    * and the subscriptions' timers can be paused individually.
    */
   Stream<T> timeout(Duration timeLimit, {void onTimeout(EventSink<T> sink)}) {
-    StreamController<T> controller;
+    _StreamControllerBase<T> controller;
     // The following variables are set on listen.
     StreamSubscription<T> subscription;
     Timer timer;
@@ -1491,8 +1490,7 @@ abstract class Stream<T> {
       timer.cancel();
       assert(controller is _StreamController ||
           controller is _BroadcastStreamController);
-      dynamic eventSink = controller;
-      eventSink._addError(error, stackTrace); // Avoid Zone error replacement.
+      controller._addError(error, stackTrace); // Avoid Zone error replacement.
       timer = zone.createTimer(timeLimit, timeout);
     }
 
@@ -2018,6 +2016,7 @@ abstract class StreamTransformer<S, T> {
    *     (stream) => stream.transform(utf8.decoder).transform(LineSplitter()));
    * ```
    */
+  @Since("2.1")
   factory StreamTransformer.fromBind(Stream<T> Function(Stream<S>) bind) =
       _StreamBindTransformer<S, T>;
 
